package uk.co.martinpearman.b4a.irdamanager;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

@Author("Martin Pearman")
@ShortName("IrdaManager")
@Version(0.02f)
public class IrdaManager {

	/**
	 * Converts a String of Hex IR codes into a String of comma separated decimal codes suitable for use with WriteIrSend.
	 * Separator denotes the separator character used in the HexData input.
	 */
	public static String Hex2Dec(String HexData, String Separator){
		List<String> hexData = new ArrayList<String>(Arrays.asList(HexData.split(Separator)));
		hexData.remove(0); // dummy
		int frequency = Integer.parseInt(hexData.remove(0), 16); // frequency
		hexData.remove(0); // seq1
		hexData.remove(0); // seq2
	
		for (int i = 0, count=hexData.size(); i < count; i++) {
			hexData.set(i, Integer.toString(Integer.parseInt(hexData.get(i), 16)));
		}
	
		frequency = (int) (1000000 / (frequency * 0.241246));
		hexData.add(0, Integer.toString(frequency));
	
		StringBuilder decData=new StringBuilder();
		for(int i=0, count=hexData.size(), lastIndex=count-1; i<count; i++){
			decData.append(hexData.get(i));
			if(i!=lastIndex){
				decData.append(",");
			}
		}
		
		return decData.toString();
	}
	
	/**
	 * IrdaManager exposes the hidden Samsung IrdaManager API to B4A.
	 * http://www.haxd.me/index.php/2013/03/reversing-samsungs-irdamanager/
	 * https://github.com/rngtng/IrDude/blob/master/src/com/rngtng/irdude/MainActivity.java
	 */
	public static void LIBRARY_DOC(){}
	
	private Object mIrdaService=null;
	private Method mIrWrite=null;
	
	/**
	 * Initialize the IrdaManager.
	 * Returns True if successfully initialized, otherwise False.
	 * If initialization fails the B4A LastException property will contain the related Exception.
	 */
	public boolean Initialize(BA pBA){
		boolean success=false;
		mIrdaService=pBA.context.getSystemService("irda");
		Class<?> irdaServiceClass=mIrdaService.getClass();
		Class<?>[] p={String.class};
		try {
			mIrWrite=irdaServiceClass.getMethod("write_irsend", p);
			success=true;
		} catch (SecurityException e) {
			e.printStackTrace();
			pBA.setLastException(e);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			pBA.setLastException(e);
		}
		return success;
	}
	
	/**
	 * Returns whether or not the IrdaManager is initialized.
	 */
	public boolean IsInitialized(){
		return mIrWrite==null?false:true;
	}
	
	/**
	 * Sends Data using the IrdaManager.
	 * Returns True if successfully sent, otherwise False.
	 * If send fails the B4A LastException property will contain the related Exception.
	 */
	public boolean WriteIrSend(BA pBA, String DecimalData){
		boolean success =false;
		try {
			mIrWrite.invoke(mIrdaService, DecimalData);
			success=true;
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			pBA.setLastException(e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			pBA.setLastException(e);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			pBA.setLastException(e);
		}
		return success;
	}
}
